'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var fiber = require('@react-three/fiber');
var drei = require('@react-three/drei');
var wouter = require('wouter');
var getUuid = _interopDefault(require('uuid-by-string'));
var THREE = _interopDefault(require('three'));

var GOLDENRATIO = 1.61803398875;
var ThreeCover = function ThreeCover(images) {
  return (
    // @ts-ignore
    React__default.createElement(fiber.Canvas, {
      gl: {
        alpha: false
      },
      dpr: [1, 1.5],
      camera: {
        fov: 70,
        position: [0, 2, 15]
      }
    }, React__default.createElement("color", {
      attach: "background",
      args: ['#191920']
    }), React__default.createElement("fog", {
      attach: "fog",
      args: ['#191920', 0, 15]
    }), React__default.createElement(drei.Environment, {
      preset: "city"
    }), React__default.createElement("group", {
      position: [0, -0.5, 0]
    },
    // @ts-ignore
    React__default.createElement(Frames, {
      images: images
    }), React__default.createElement("mesh", {
      rotation: [-Math.PI / 2, 0, 0],
      position: [0, 0, 0]
    }, React__default.createElement("planeGeometry", {
      args: [50, 50]
    }),
    // @ts-ignore
    React__default.createElement(drei.MeshReflectorMaterial, {
      blur: [300, 100],
      resolution: 2048,
      mixBlur: 1,
      mixStrength: 40,
      roughness: 1,
      depthScale: 1.2,
      minDepthThreshold: 0.4,
      maxDepthThreshold: 1.4,
      color: "#050505",
      metalness: 0.5
    }))))
  );
};
function Frames(images, q, p) {
  var _images$;
  if (q === void 0) {
    q = new THREE.Quaternion();
  }
  if (p === void 0) {
    p = new THREE.Vector3();
  }
  var ref = React.useRef();
  var clicked = React.useRef();
  var _useRoute = wouter.useRoute('/item/:id'),
    params = _useRoute[1];
  var _useLocation = wouter.useLocation(),
    setLocation = _useLocation[1];
  React.useEffect(function () {
    // @ts-ignore
    clicked.current = ref.current.getObjectByName(params == null ? void 0 : params.id);
    if (clicked.current) {
      // @ts-ignore
      clicked.current.parent.updateWorldMatrix(true, true);
      // @ts-ignore
      clicked.current.parent.localToWorld(p.set(0, GOLDENRATIO / 2, 1.25));
      // @ts-ignore
      clicked.current.parent.getWorldQuaternion(q);
    } else {
      p.set(0, 0, 5.5);
      q.identity();
    }
  });
  fiber.useFrame(function (state) {
    state.camera.position.lerp(p, 0.025);
    state.camera.quaternion.slerp(q, 0.025);
  });
  return React__default.createElement("group", {
    // @ts-ignore
    ref: ref,
    onClick: function onClick(e) {
      return e.stopPropagation(), setLocation(clicked.current === e.object ? '/' : '/item/' + e.object.name);
    },
    onPointerMissed: function onPointerMissed() {
      return setLocation('/');
    }
  }, React__default.createElement(Frame, Object.assign({
    key: (_images$ = images[0]) == null ? void 0 : _images$.url
  }, images[0])));
}
function Frame(url, c) {
  if (c === void 0) {
    c = new THREE.Color();
  }
  var _useState = React.useState(false),
    hovered = _useState[0],
    hover = _useState[1];
  var _useState2 = React.useState(function () {
      return Math.random();
    }),
    rnd = _useState2[0];
  var image = React.useRef();
  var frame = React.useRef();
  var name = getUuid(url.toString());
  drei.useCursor(hovered);
  fiber.useFrame(function (state) {
    // @ts-ignore
    image.current.material.zoom = 2 + Math.sin(rnd * 10000 + state.clock.elapsedTime / 3) / 2;
    // @ts-ignore
    image.current.scale.x = THREE.MathUtils.lerp(
    // @ts-ignore
    image.current.scale.x, 0.85 * (hovered ? 0.85 : 1), 0.1);
    // @ts-ignore
    image.current.scale.y = THREE.MathUtils.lerp(
    // @ts-ignore
    image.current.scale.y, 0.9 * (hovered ? 0.905 : 1), 0.1);
    // @ts-ignore
    frame.current.material.color.lerp(c.set(hovered ? 'orange' : 'white'), 0.1);
  });
  for (var _len = arguments.length, props = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    props[_key - 2] = arguments[_key];
  }
  return React__default.createElement("group", Object.assign({}, props), React__default.createElement("mesh", {
    name: name,
    onPointerOver: function onPointerOver(e) {
      return e.stopPropagation(), hover(true);
    },
    onPointerOut: function onPointerOut() {
      return hover(false);
    },
    scale: [1, GOLDENRATIO, 0.05],
    position: [0, GOLDENRATIO / 2, 0]
  }, React__default.createElement("boxGeometry", null), React__default.createElement("meshStandardMaterial", {
    color: "#151515",
    metalness: 0.5,
    roughness: 0.5,
    envMapIntensity: 2
  }), React__default.createElement("mesh", {
    // @ts-ignore
    ref: frame,
    raycast: function raycast() {
      return null;
    },
    scale: [0.9, 0.93, 0.9],
    position: [0, 0, 0.2]
  }, React__default.createElement("boxGeometry", null), React__default.createElement("meshBasicMaterial", {
    toneMapped: false,
    fog: false
  })), React__default.createElement(drei.Image, {
    raycast: function raycast() {
      return null;
    },
    // @ts-ignore
    ref: image,
    position: [0, 0, 0.7],
    url: url
  })), React__default.createElement(drei.Text, {
    maxWidth: 0.1,
    anchorX: "left",
    anchorY: "top",
    position: [0.55, GOLDENRATIO, 0],
    fontSize: 0.025
  }, name.split('-').join(' ')));
}

exports.ThreeCover = ThreeCover;
//# sourceMappingURL=three-cover.cjs.development.js.map
