import React, { useRef, useEffect, useState } from 'react';
import { Canvas, useFrame } from '@react-three/fiber';
import { Environment, MeshReflectorMaterial, useCursor, Image, Text } from '@react-three/drei';
import { useRoute, useLocation } from 'wouter';
import getUuid from 'uuid-by-string';
import THREE from 'three';

var GOLDENRATIO = 1.61803398875;
var ThreeCover = function ThreeCover(images) {
  return (
    // @ts-ignore
    React.createElement(Canvas, {
      gl: {
        alpha: false
      },
      dpr: [1, 1.5],
      camera: {
        fov: 70,
        position: [0, 2, 15]
      }
    }, React.createElement("color", {
      attach: "background",
      args: ['#191920']
    }), React.createElement("fog", {
      attach: "fog",
      args: ['#191920', 0, 15]
    }), React.createElement(Environment, {
      preset: "city"
    }), React.createElement("group", {
      position: [0, -0.5, 0]
    },
    // @ts-ignore
    React.createElement(Frames, {
      images: images
    }), React.createElement("mesh", {
      rotation: [-Math.PI / 2, 0, 0],
      position: [0, 0, 0]
    }, React.createElement("planeGeometry", {
      args: [50, 50]
    }),
    // @ts-ignore
    React.createElement(MeshReflectorMaterial, {
      blur: [300, 100],
      resolution: 2048,
      mixBlur: 1,
      mixStrength: 40,
      roughness: 1,
      depthScale: 1.2,
      minDepthThreshold: 0.4,
      maxDepthThreshold: 1.4,
      color: "#050505",
      metalness: 0.5
    }))))
  );
};
function Frames(images, q, p) {
  var _images$;
  if (q === void 0) {
    q = new THREE.Quaternion();
  }
  if (p === void 0) {
    p = new THREE.Vector3();
  }
  var ref = useRef();
  var clicked = useRef();
  var _useRoute = useRoute('/item/:id'),
    params = _useRoute[1];
  var _useLocation = useLocation(),
    setLocation = _useLocation[1];
  useEffect(function () {
    // @ts-ignore
    clicked.current = ref.current.getObjectByName(params == null ? void 0 : params.id);
    if (clicked.current) {
      // @ts-ignore
      clicked.current.parent.updateWorldMatrix(true, true);
      // @ts-ignore
      clicked.current.parent.localToWorld(p.set(0, GOLDENRATIO / 2, 1.25));
      // @ts-ignore
      clicked.current.parent.getWorldQuaternion(q);
    } else {
      p.set(0, 0, 5.5);
      q.identity();
    }
  });
  useFrame(function (state) {
    state.camera.position.lerp(p, 0.025);
    state.camera.quaternion.slerp(q, 0.025);
  });
  return React.createElement("group", {
    // @ts-ignore
    ref: ref,
    onClick: function onClick(e) {
      return e.stopPropagation(), setLocation(clicked.current === e.object ? '/' : '/item/' + e.object.name);
    },
    onPointerMissed: function onPointerMissed() {
      return setLocation('/');
    }
  }, React.createElement(Frame, Object.assign({
    key: (_images$ = images[0]) == null ? void 0 : _images$.url
  }, images[0])));
}
function Frame(url, c) {
  if (c === void 0) {
    c = new THREE.Color();
  }
  var _useState = useState(false),
    hovered = _useState[0],
    hover = _useState[1];
  var _useState2 = useState(function () {
      return Math.random();
    }),
    rnd = _useState2[0];
  var image = useRef();
  var frame = useRef();
  var name = getUuid(url.toString());
  useCursor(hovered);
  useFrame(function (state) {
    // @ts-ignore
    image.current.material.zoom = 2 + Math.sin(rnd * 10000 + state.clock.elapsedTime / 3) / 2;
    // @ts-ignore
    image.current.scale.x = THREE.MathUtils.lerp(
    // @ts-ignore
    image.current.scale.x, 0.85 * (hovered ? 0.85 : 1), 0.1);
    // @ts-ignore
    image.current.scale.y = THREE.MathUtils.lerp(
    // @ts-ignore
    image.current.scale.y, 0.9 * (hovered ? 0.905 : 1), 0.1);
    // @ts-ignore
    frame.current.material.color.lerp(c.set(hovered ? 'orange' : 'white'), 0.1);
  });
  for (var _len = arguments.length, props = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    props[_key - 2] = arguments[_key];
  }
  return React.createElement("group", Object.assign({}, props), React.createElement("mesh", {
    name: name,
    onPointerOver: function onPointerOver(e) {
      return e.stopPropagation(), hover(true);
    },
    onPointerOut: function onPointerOut() {
      return hover(false);
    },
    scale: [1, GOLDENRATIO, 0.05],
    position: [0, GOLDENRATIO / 2, 0]
  }, React.createElement("boxGeometry", null), React.createElement("meshStandardMaterial", {
    color: "#151515",
    metalness: 0.5,
    roughness: 0.5,
    envMapIntensity: 2
  }), React.createElement("mesh", {
    // @ts-ignore
    ref: frame,
    raycast: function raycast() {
      return null;
    },
    scale: [0.9, 0.93, 0.9],
    position: [0, 0, 0.2]
  }, React.createElement("boxGeometry", null), React.createElement("meshBasicMaterial", {
    toneMapped: false,
    fog: false
  })), React.createElement(Image, {
    raycast: function raycast() {
      return null;
    },
    // @ts-ignore
    ref: image,
    position: [0, 0, 0.7],
    url: url
  })), React.createElement(Text, {
    maxWidth: 0.1,
    anchorX: "left",
    anchorY: "top",
    position: [0.55, GOLDENRATIO, 0],
    fontSize: 0.025
  }, name.split('-').join(' ')));
}

export { ThreeCover };
//# sourceMappingURL=three-cover.esm.js.map
